/*
 * Name:    vra_relation_update_MSSQL.sql for MS-SQL
 *
 * Purpose: Corrects the Inscribe source write back issue for data already
 *          cataloged.
 *          Uses IREntityChanges and DTValueToEntityMap to determine the
 *          missing mapping information in the source data, then inserts the
 *          missing relationships.
 *
 * Date:    2004/02/27
 *
 * Copyright (c) 2004, Luna Imaging, Inc.  All Rights Reserved
 *
*/

/* Should the Spool Stored Procedure Exist - Delete It */
if exists (select * from sysobjects where id = object_id(N'[dbo].[Spool]') and OBJECTPROPERTY(id, N'IsProcedure') = 1)
drop procedure [dbo].[Spool]
go



/* Create Spool Stored Procedure */
create procedure Spool(
@Filename nvarchar(500),
@SQL nvarchar(3500))
as
declare @TSQL nvarchar(4000)
set @TSQL = 'osql -E -n -w5000 -d Insightvra -Q"' + @SQL + '" -o ' + @FileName
exec master.dbo.xp_cmdshell @TSQL, no_output
return 0
go





Declare missingMapping CURSOR FOR
  /*
   * Determine the list of EntityType/Entities matching the following criteria:
   * 1) Dependent EntityType Entities
   * 2) Have been committed for the source data
   * 3) Dependent EntityTypes which only contain a single mapping field
   *    (i.e. Notes)
  */
  Select EntityTypeID, EntityID
  from IREntityChanges EC1
  where changecategory = 2
  AND committedtosourcedata = 1
  AND changetype = 1
  AND entityid not in (select distinct entityid 
                       from IREntityChanges EC3
                       where (changecategory=1) and (changetype=1)
                       and EC3.entitytypeid = EC1.entitytypeid)
  AND entityid not in (select distinct entityid 
                       from IREntityChanges EC4
                       where (changecategory=4) and (changetype=2)
                       and EC4.entitytypeid = EC1.entitytypeid)
  AND 1 = (select count(*) from ISEntityFields, ISEntityTypes 
           where ISEntityFields.EntityTypeID = ISEntityTypes.EntityTypeID 
           and ISEntityTypes.IsDependent = 1 
           and ISEntityFields.EntityTypeID = EC1.EntityTypeID)
  Group by entitytypeid, entityid
  ORDER BY entitytypeid, entityid; 


OPEN missingMapping;

Declare @vFieldID           varchar(255);

Declare @vSourceTable       varchar(255);
Declare @vSourceColumnID    varchar(255);
Declare @vSourceColumnField varchar(255);

DECLARE @CURRID INT;

Declare @entitytypeid       int;
Declare @entityid           int;


  Print space(10);
  Print space(10) + '-- ***************************************';
  Print space(10) + '-- Start time: ' + CONVERT(CHAR(24),GETDATE(),113);

  Print space(10);
    

  -- Get all the missing relationships

    WHILE @@FETCH_STATUS <> -1
    BEGIN
    
    FETCH NEXT FROM missingMapping
    INTO @entitytypeid, @entityid;
    
    --Print CONVERT(CHAR(10), @entitytypeid) + ',' + CONVERT(CHAR(10), @entityid);

    -- Determine the field associated with the EntityType
    set  @vFieldID = (Select FieldID from ISEntityFields
    where EntityTypeID = @entitytypeid);

    -- Determine the table name
    set @vSourceTable =  (select ISTables.TableName from IRFields, ISTables
    where IRFields.TableID = ISTables.TableID
    and IRFields.FieldID = @vFieldID);

    -- Determine the increment column name
    set @vSourceColumnID = (select distinct StartFieldName from ISJoins
    where StartTableID in
    (select TableID from IRFields where FieldID = @vFieldID));

    -- Determine the source field name
    set @vSourceColumnField = (Select FieldName from IRFields
    where fieldid = @vFieldID);

    -- Print 'insert into  ' + @vSourceTable
    --  + '(' + @vSourceColumnID + ', ' + @vSourceColumnField + ') Values('
    --  --+ @entityid + ', NULL)';
    --  + CONVERT(CHAR(10), @entityid) + ', NULL);';
    
    declare @TSQL nvarchar(4000)
    set @TSQL = 'insert into  ' + @vSourceTable
      + '(' + @vSourceColumnID + ', ' + @vSourceColumnField + ') Values('
      + CONVERT(CHAR(10), @entityid) + ', NULL);';
    
    
    exec Spool 'vra_sql_insert.sql' , @TSQL

  END


  /* Clean Up */
  close missingMapping;
  Deallocate missingMapping;
  
  /* Delete It */
  if exists (select * from sysobjects where id = object_id(N'[dbo].[Spool]') and OBJECTPROPERTY(id, N'IsProcedure') = 1)
  drop procedure [dbo].[Spool]
  go

  Print space(10);
  Print space(10) + '-- End time: ' + CONVERT(CHAR(24),GETDATE(),113);
  Print space(10) + '-- ***************************************';
  Print space(10);



